package core.engine.terrain;

import core.engine.Loader;
import core.engine.entity.Entity;
import core.engine.model.HexagonModel;
import core.engine.model.RawModel;
import core.engine.model.TexturedModel;
import core.engine.textures.ModelTexture;
import utils.vectors.Vector3f;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Terrain {


    private final List<TerrainTile> terrainTiles;
    private static Vector3f colors[] = new Vector3f[]{
            new Vector3f(1f, 0f, 0.86f),
            new Vector3f( 0.05f, 0.3f, 0f),
            //new Vector3f(1f, 0.5f, 0),
            //new Vector3f(0, 0.84f, 1f),
            //new Vector3f(1, 1, 0)
    };

    private Terrain(List<TerrainTile> terrainTiles) {
        this.terrainTiles = terrainTiles;
    }

    public static Terrain generateTerrain(Loader loader) {
        HexagonModel hexagonModel = new HexagonModel();
        RawModel hexagonRawModel = loader.loadToVAO(hexagonModel.getVertices(), hexagonModel.getTextureCoords(), hexagonModel.getNormals(), hexagonModel.getIndices());
        ModelTexture hexagonTexture = new ModelTexture(loader.loadTexture("white"));
        TexturedModel hexagontexturedModel = new TexturedModel(hexagonRawModel, hexagonTexture);

        ArrayList<TerrainTile> generatedTerrainTiles = new ArrayList<>();
        TerrainTile[][] terrainTilesGrid = new TerrainTile[18][];

        for(int y = -9; y<9; y++) {
            for(int x=-16; x<16; x++) {
                double keepSample = Math.random();
                if(keepSample > 0.1) {
                    float xPos = x * 1.7319988f;
                    if(y % 2 == 0) {
                        xPos -= 0.866f;
                    }

                    TerrainTile terrainTile = new TerrainTile(hexagonRawModel, new Vector3f(xPos , 0, y * -1.4999994f), 0, 0, 0,1);
                    generatedTerrainTiles.add(terrainTile);
                }
            }
        }

        List<TerrainTile> selectedTerrainTiles = new ArrayList<>();
        List<Integer> usedIndices = new ArrayList<>();
        for(int i=0; i<colors.length; i++) {
            int randomIndex = (int) (Math.random() * generatedTerrainTiles.size());

            while(usedIndices.contains(randomIndex)) {
                randomIndex =  (int) (Math.random() * generatedTerrainTiles.size());
            }

            usedIndices.add(randomIndex);
            generatedTerrainTiles.get(i).setColor(colors[i]);
            selectedTerrainTiles.add(generatedTerrainTiles.get(i));
        }

        System.out.println(usedIndices);

        for(int i=0; i<generatedTerrainTiles.size(); i++) {
            TerrainTile nearestTerrainTile = Terrain.getNearestTerrainTile(selectedTerrainTiles, generatedTerrainTiles.get(i));
            generatedTerrainTiles.get(i).setColor(nearestTerrainTile.getColor());
        }

        for(int i=0; i<generatedTerrainTiles.size(); i++) {
            if(generatedTerrainTiles.get(i).getColor().length() == 0) {
                System.out.println("Null");
            }
        }
        return new Terrain(generatedTerrainTiles);
    }

    private static TerrainTile getNearestTerrainTile(List<TerrainTile> terrainTiles, TerrainTile currentTerrainTile) {
        TerrainTile nearestTerrainTile = null;
        float nearestDistance = Float.MAX_VALUE;
        for(TerrainTile terrainTile : terrainTiles) {
            if(nearestTerrainTile == null) {
                nearestTerrainTile = terrainTile;
                nearestDistance = Vector3f.sub(terrainTile.getPosition(), currentTerrainTile.getPosition(), null).length();
            } else {
                float distance = Vector3f.sub(terrainTile.getPosition(), currentTerrainTile.getPosition(), null).length();
                if(distance < nearestDistance) {
                    nearestDistance = distance;
                    nearestTerrainTile = terrainTile;
                }
            }
        }

        System.out.println(nearestDistance);

        return nearestTerrainTile;
    }

    public List<TerrainTile> getTerrainTiles() {
        return terrainTiles;
    }
}
